/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.model.memory.loader;

import java.nio.file.Path;
import java.nio.file.Paths;

import ru.ispras.microtesk.SysUtils;

/**
 * The {@link ParametersLoader} class is provided methods for reading model memory parameters
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class ParametersLoader {
  public static final String ARCH_PATH_PART = "arch";
  public static final String SETTINGS_PATH_PART = "tools_settings";

  // File contains model memory regions
  public static final String MEMORY_REGIONS_FILE = "memory_regions.json";
  // The file contains an abstract description of the structure of the element in the model memory
  public static final String MEMORY_STRUCTURE_FILE = "memory_structure.json";

  public static Path getParametersPath(final String modelName) {
    return Paths.get(SysUtils.getHomeDir(), ARCH_PATH_PART, modelName, SETTINGS_PATH_PART);
  }

  public static Path getRegionsPath(final String modelName) {
    return ParametersLoader.getParametersPath(modelName).resolve(MEMORY_REGIONS_FILE);
  }

  public static Path getStructurePath(final String modelName) {
    return ParametersLoader.getParametersPath(modelName).resolve(MEMORY_STRUCTURE_FILE);
  }
}
