/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.model.tracer;

import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.microtesk.test.template.ConcreteCall;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class RecordFactory {
  private static RecordFormatType record = RecordFormatType.BASE;

  public static void setRecordType(final RecordFormatType record) {
    RecordFactory.record = record;
  }

  public static RecordFormatType getRecordType() {
    return RecordFactory.record;
  }

  public static Record newInstruction(final ConcreteCall call, final int cpu) {
    switch (record) {
      case BASE:
        return Record.newInstruction(call, cpu);
      case ARK:
        return ArkRecord.newInstruction(call, cpu);
      default: // BASE
        return Record.newInstruction(call, cpu);
    }
  }

  public static Record newMemoryAccess(final long address, final BitVector data,
      final boolean isWrite) {
    switch (record) {
      case BASE:
        return Record.newMemoryAccess(address, data, isWrite);
      case ARK:
        return Record.newEmptyRecord();
      default: // BASE
        return Record.newMemoryAccess(address, data, isWrite);
    }
  }

  public static Record newRegisterWrite(final String register, final BitVector value) {
    switch (record) {
      case BASE:
        return Record.newRegisterWrite(register, value);
      case ARK:
        return ArkRecord.newRegisterWrite(register, value);
      default: // BASE
        return Record.newRegisterWrite(register, value);
    }
  }

}
