/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.model.tracer;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public enum RecordFormatType {
  /**
   * Trace format in a TARMAC like style.
   */
  BASE(RecordFormatType.BASE_TYPE_NAME),

  /**
   * Trace format in a Ark(Panda) style.
   */
  ARK(RecordFormatType.ARK_TYPE_NAME);

  public final static String BASE_TYPE_NAME = "base";
  public final static String ARK_TYPE_NAME = "ark";
  public final static String PANDA_TYPE_NAME = "panda";

  private final String formatName;

  private RecordFormatType(final String formatName) {
    InvariantChecks.checkNotNull(formatName);

    this.formatName = formatName;
  }

  public String getName() {
    return formatName;
  }

  public static RecordFormatType getType(final String element) {
    switch (element) {
      case BASE_TYPE_NAME:
        return BASE;

      case ARK_TYPE_NAME:
        return ARK;

      case PANDA_TYPE_NAME:
        return ARK;

      default:
        return ARK;
    }
  }

}
