/*
 * Copyright 2021-2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools;

import java.io.File;

import ru.ispras.castle.util.Logger;
import ru.ispras.microtesk.SysUtils;
import ru.ispras.microtesk.model.Model;
import ru.ispras.microtesk.options.Option;
import ru.ispras.microtesk.options.Options;

/**
 * The {@link ToolUtils} class provides utility methods for tools to interact with the environment.
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class ToolUtils {

  /**
   * Returns the {@link Model}.
   *
   * @param modelName the model name.
   * @return the {@link Model}.
   */
  public static Model loadModel(final String modelName) {
    try {
      return SysUtils.loadModel(modelName);
    } catch (final Exception e) {
      Logger.error("Failed to load the %s model. Reason: %s.", modelName, e.getMessage());
      return null;
    }
  }

  public static String getOutDir(final Options options) {
    return options.hasValue(Option.OUTPUT_DIR) ? options.getValueAsString(Option.OUTPUT_DIR)
        : SysUtils.getHomeDir();
  }

  public static String getOutDir(final Options options, final String outDirName) {
    return options.hasValue(Option.OUTPUT_DIR) ? options.getValueAsString(Option.OUTPUT_DIR)
        : String.format("%s%s%s", SysUtils.getHomeDir(), File.separator, outDirName);
  }

  public static String getFinalInstruction(final Options options) {
    return options.hasValue(Option.FINAL_INSTRUCTION)
        ? options.getValueAsString(Option.FINAL_INSTRUCTION)
        : null;
  }

  public static String getOutDir(final String pathOutDir) {
    return pathOutDir.isEmpty() ? SysUtils.getHomeDir() : pathOutDir;
  }
}
