/*
 * Copyright 2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.adapter;

import java.io.File;
import java.io.FileNotFoundException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import javax.json.JsonObject;
import javax.json.JsonReader;

import ru.ispras.castle.util.Logger;
import ru.ispras.microtesk.utils.JsonConfig;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class AdapterSettings {
  // The name of the file with the oracle settings
  public static final String ADAPTER_SETTINGS_FILE = "trace_adapter_settings.json";

  public static final String ADAPTER_EVENTS_FILE_TAG = "events_file_suffix";

  // Trace name postfix for events
  public static final String ADAPTER_EVENTS_JSON_FILE_SUFFIX_DEFAULT = "_trace.json";

  public static String getAdapterEventSuffix(final String inputResourceDir) {
    final String adapterSettingsFile =
        inputResourceDir + File.separator + AdapterSettings.ADAPTER_SETTINGS_FILE;

    String adapterEventSuffix;

    final Path adapterSettingsPath = Paths.get(adapterSettingsFile);
    try {
      if (!Files.exists(adapterSettingsPath)) {
        final String errorMessage =
            String.format("[AdapterSettings] Files %s is not exist.", adapterSettingsPath);
        Logger.error(errorMessage);
        throw new FileNotFoundException(errorMessage);
      }

      JsonReader jsonReader = JsonConfig.getJsonReader(adapterSettingsPath.toString());
      JsonObject jsonObject = jsonReader.readObject();

      if (!jsonObject.containsKey(AdapterSettings.ADAPTER_EVENTS_FILE_TAG)) {
        Logger.warning("[AdapterSettings] The file '%s' does not contain the tag '%s'.",
            AdapterSettings.ADAPTER_SETTINGS_FILE, AdapterSettings.ADAPTER_EVENTS_FILE_TAG);
        adapterEventSuffix = AdapterSettings.ADAPTER_EVENTS_JSON_FILE_SUFFIX_DEFAULT;
      } else {
        adapterEventSuffix = jsonObject.getString(AdapterSettings.ADAPTER_EVENTS_FILE_TAG);
      }
    } catch (FileNotFoundException e) {
      adapterEventSuffix = AdapterSettings.ADAPTER_EVENTS_JSON_FILE_SUFFIX_DEFAULT;
    }

    return adapterEventSuffix;
  }
}
