/*
 * Copyright 2021-2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.adapter.trace;

import java.util.Map;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class ConcreteEvent {
  private Map<EventElementType, String> eventElements;
  private EventType eventType;

  public ConcreteEvent(final EventType eventType, final Map<EventElementType, String> eventElements) {
    InvariantChecks.checkNotNull(eventType);
    InvariantChecks.checkNotNull(eventElements);

    this.eventType = eventType;
    this.eventElements = eventElements;
  }

  public EventType getEventType() {
    return eventType;
  }

  public Map<EventElementType, String> getEventElements() {
    return eventElements;
  }

  public String getElement(final EventElementType key) {
    return eventElements.get(key);
  }

  @Override
  public String toString() {
    final StringBuffer buffer = new StringBuffer();
    buffer.append(String.format("(%s: %s) Elements: ", ConcreteEvent.class.getSimpleName(),
        eventType.name()));

    boolean separator = false;
    for (EventElementType element : eventType.getElementsRegExpList()) {
      buffer.append(separator ? ", " : "").append("[").append(element).append(" - \'")
          .append(eventElements.get(element)).append("\']");
      separator = true;
    }

    return buffer.toString();
  }
}
