/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.adapter.trace;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * The {@code EventElementType} {@code enum} describes event element types.
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public enum EventElementType {
  THREAD_IDENTIFIER("ThreadIdentifier"),
  SOURCE("Source"),
  PC("PC"),
  INSTRUCTION("Instruction"),
  REGISTER("Register"),
  REGISTER_VALUE("RegisterValue"),
  REGISTER_TYPE("RegisterType"),
  FRAME_ENTRY("FrameEntry"),
  FRAME_EXIT("FrameExit");

  private final String elementName;

  private EventElementType(final String elementName) {
    InvariantChecks.checkNotNull(elementName);

    this.elementName = elementName;
  }

  public String getName() {
    return elementName;
  }

  public static EventElementType getType(final String element) {
    switch (element) {
      case "ThreadIdentifier":
        return THREAD_IDENTIFIER;

      case "PC":
        return PC;

      case "Source":
        return SOURCE;

      case "Instruction":
        return INSTRUCTION;

      case "Register":
        return REGISTER;

      case "RegisterType":
        return REGISTER_TYPE;

      case "RegisterValue":
        return REGISTER_VALUE;

      case "FrameEntry":
        return FRAME_ENTRY;

      case "FrameExit":
        return FRAME_EXIT;

      default:
        return null;
    }
  }
}