/*
 * Copyright 2021-2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.adapter.trace;

import java.util.List;
import java.util.Map;

import javax.json.Json;
import javax.json.JsonArrayBuilder;
import javax.json.JsonObjectBuilder;
import javax.json.JsonWriter;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.microtesk.utils.JsonConfig;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class EventListWriter {
  private JsonWriter jsonWriter;

  public EventListWriter(final String fileName) {
    InvariantChecks.checkNotNull(fileName);

    jsonWriter = JsonConfig.getJsonWriter(fileName);
  }

  public boolean write(final List<ConcreteEvent> inputStructure) {
    InvariantChecks.checkNotNull(inputStructure);

    final JsonArrayBuilder jsonArrayBuilder = Json.createArrayBuilder();
    for(final ConcreteEvent event: inputStructure) {
      final JsonObjectBuilder objectBuilder = Json.createObjectBuilder();
      objectBuilder.add(EventType.EVENT_TYPE_IDENTIFIER, event.getEventType().name());

      Map<EventElementType, String> eventElements = event.getEventElements();
      eventElements.forEach((key, value) -> objectBuilder.add(key.getName(), value));

      jsonArrayBuilder.add(objectBuilder);
    }
    jsonWriter.writeArray(jsonArrayBuilder.build());

    return true;
  }

  public void close() {
    JsonConfig.close(jsonWriter);
  }

  public static void saveEventsToJson(final List<ConcreteEvent> arrayElements,
      final String outputJsonFile) {
    // Logger.debug("The output file for trace: %s", outputJsonFile);
    final EventListWriter eventListWriter = new EventListWriter(outputJsonFile);
    eventListWriter.write(arrayElements);
    eventListWriter.close();
  }
}