/*
 * Copyright 2021-2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.adapter.trace;

import java.util.List;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public enum EventType {
  INSTRUCTION("instruction execution start", List.of(EventElementType.THREAD_IDENTIFIER,
                                                     EventElementType.PC,
                                                     EventElementType.INSTRUCTION)),
  MEMORY("assignment to global memory", List.of(EventElementType.THREAD_IDENTIFIER,
                                                EventElementType.REGISTER,
                                                EventElementType.REGISTER_TYPE,
                                                EventElementType.REGISTER_VALUE)),
  FRAME_ENTRY("frame entry", List.of(EventElementType.THREAD_IDENTIFIER,
                                     EventElementType.FRAME_ENTRY)),
  FRAME_EXIT("frame exit", List.of(EventElementType.THREAD_IDENTIFIER,
                                   EventElementType.FRAME_EXIT)),
  FRAME_SETTING("setting the current frame", List.of()),
  FRAME_ASSIGNMENT("assignment to the current frame", List.of()),
  EXCEPTION("raising an exception", List.of()),
  INFO("information", List.of());

  public static final String EVENT_TYPE_IDENTIFIER = "EventType";

  private final String fullName;
  private final List<EventElementType> elementsRegExpList;

  private EventType(final String fullName, final List<EventElementType> elements) {
    InvariantChecks.checkNotNull(fullName);

    this.fullName = fullName;
    this.elementsRegExpList = elements;
  }

  public String getFullName() {
    return fullName;
  }

  public List<EventElementType> getElementsRegExpList() {
    return elementsRegExpList;
  }
}
