/*
 * Copyright 2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.isa;

import ru.ispras.castle.util.Logger;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.microtesk.model.Model;
import ru.ispras.microtesk.model.metadata.MetaModel;
import ru.ispras.microtesk.model.metadata.MetaOperation;
import ru.ispras.microtesk.options.Options;
import ru.ispras.microtesk.tools.ToolUtils;

/**
 * The {@code IsaListing} creates a list of a model operations and prints it out.
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class IsaListing {

  public static boolean createIsaList(final Options options,final String modelName) {
    InvariantChecks.checkNotNull(options);
    InvariantChecks.checkNotNull(modelName);
    Logger.setDebug(true);

    Logger.message("Starting the ISA listing tool.");
    Logger.message("Model name: %s", modelName);

    final Model model = ToolUtils.loadModel(modelName);
    final MetaModel metaModel = model.getMetaData();

    final Iterable<MetaOperation> operationsIterator = metaModel.getOperations();

    int operationCount = 1;
    for (MetaOperation operation : operationsIterator) {
      Logger.message("%d. %s", operationCount, operation.getName());
      //Logger.message("      Arguments: %s", operation.getArguments());

      operationCount++;
    }
    Logger.message("Total operations: %d", --operationCount);

    return true;
  }
}
