/*
 * Copyright 2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle;

import java.io.File;
import java.io.FileNotFoundException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import javax.json.JsonObject;
import javax.json.JsonReader;

import ru.ispras.castle.util.Logger;
import ru.ispras.microtesk.utils.JsonConfig;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class OracleSettings {
  // The name of the file with the oracle settings
  public static final String ORACLE_SETTINGS_FILE = "oracle_settings.json";

  public static final String ORACLE_LOG_FILE_TAG = "oracle_log_file_name";

  public static final String ORACLE_LOG_FILE_DEFAULT = "oracle.log";

  public static String getOracleLogFileName(final String inputResourceDir) {
    final String oracleSettingsFile =
        inputResourceDir + File.separator + OracleSettings.ORACLE_SETTINGS_FILE;

    String oracleLogFileName;

    final Path oracleSettingsPath = Paths.get(oracleSettingsFile);
    try {
      if (!Files.exists(oracleSettingsPath)) {
        final String errorMessage =
            String.format("[OracleSettings] Files %s is not exist.", oracleSettingsPath);
        Logger.error(errorMessage);
        throw new FileNotFoundException(errorMessage);
      }

      JsonReader jsonReader = JsonConfig.getJsonReader(oracleSettingsPath.toString());
      JsonObject jsonObject = jsonReader.readObject();

      if (!jsonObject.containsKey(OracleSettings.ORACLE_LOG_FILE_TAG)) {
        Logger.warning("[OracleSettings] The file '%s' does not contain the tag '%s'.",
            OracleSettings.ORACLE_SETTINGS_FILE, OracleSettings.ORACLE_LOG_FILE_TAG);
        oracleLogFileName = OracleSettings.ORACLE_LOG_FILE_DEFAULT;
      } else {
        oracleLogFileName = jsonObject.getString(OracleSettings.ORACLE_LOG_FILE_TAG);
      }
    } catch (FileNotFoundException e) {
      oracleLogFileName = OracleSettings.ORACLE_LOG_FILE_DEFAULT;
    }

    return oracleLogFileName;
  }
}
