/*
 * Copyright 2022-2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import ru.ispras.castle.util.FileUtils;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class Statistic {
  private FileWriter logWriter;

  public Statistic(final String logFileFullName, final boolean appendText) {
    try {
      File logFileName = FileUtils.newFile(logFileFullName);
      this.logWriter = new FileWriter(logFileName, appendText);
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public Statistic(final String outputDir) {
    this(outputDir, false);
  }

  public void write(final String text) {
    try {
      logWriter.write(text);
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public void writeResult(final int traceSuccess, final int traceFail) {
    this.write(String.format("Statistics: all = %s, ok = %s, failed = %s", traceSuccess + traceFail,
        traceSuccess, traceFail, System.lineSeparator()));
  }

  public String writeSimpleResult(final String trace, final boolean result) {
    String simpleLog = String.format("Program %s - %s", FileUtils.getShortFileName(trace),
        result ? "OK" : "failed");
    this.write(String.format("%s (%s)%s", simpleLog, new java.util.Date(), System.lineSeparator()));
    return simpleLog;
  }

  public void close() {
    try {
      logWriter.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
}
