/*
 * Copyright 2021-2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle.matcher;

import java.util.List;

import ru.ispras.castle.util.Logger;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.microtesk.tools.adapter.trace.ConcreteEvent;
import ru.ispras.microtesk.tools.adapter.trace.Events;

/**
 * The {@link EventList} class returns trace events one by one.
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class EventList implements Events {
  private final List<ConcreteEvent> eventsList;

  private int index = 0;

  public EventList(final List<ConcreteEvent> eventsList) {
    InvariantChecks.checkNotNull(eventsList);

    EventTransformer eventTransformer = new EventTransformer();
    this.eventsList = eventTransformer.normalizeEvents(eventsList);
  }

  public boolean hasNext() {
    return eventsList.size() > index + 1;
  }

  public ConcreteEvent readNext() {
    return eventsList.get(index++);
  }

  public ConcreteEvent getNextEvent() {
    if (this.hasNext()) {
      return eventsList.get(index + 1);
    } else {
      return null;
    }
  }

  public ConcreteEvent getCurrentEvent() {
    return eventsList.get(index);
  }

  /**
   * @return the index
   */
  public int getIndex() {
    return index;
  }

  /**
   * @param index the index to set
   */
  public void setIndex(int index) {
    Logger.debug("[%s] Set the event index : '%s'", EventList.class.getSimpleName(), index);
    this.index = index;
  }
}
