/*
 * Copyright 2022-2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle.matcher;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public interface OracleEventMatcher {
  public static final String SUCCESSFUL_MATCH = "SUCCESS";
  public static final String FAILURE_MATCH = "FAILURE";

  /**
   * Returns {@code true} if the oracle has more events.
   *
   * @return true if the oracle has more events
   */
  public boolean hasNext();

  /**
   * Moves to the next active oracle event.
   */
  public void nextEvent();

  /**
   * Matches the event from the simulator and the event from the trace, and returns {@code true} if
   * they match.
   *
   * @return true if the event match
   */
  public boolean checkEvent();

  /**
   * Returns {@code true} if the oracle does not contain trace events to match
   *
   * @return true if the oracle does not contain trace events to match
   */
  public boolean isAllTraceEventsMatched();
}
