/*
 * Copyright 2021 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.simulator;

import java.util.List;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.microtesk.model.IsaPrimitive;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public final class IrProgram {
  private List<IsaPrimitive> instructions;
  private long instructionAddress;
  private byte[] data;
  private long dataAddress;
  private String programName;

  IrProgram(final List<IsaPrimitive> instructions, final String name) {
    InvariantChecks.checkNotNull(instructions);
    InvariantChecks.checkNotNull(name);

    this.instructions = instructions;
    this.programName = name;
    this.data = new byte[0];
  }

  IrProgram(final List<IsaPrimitive> instructions, final long instructionAddress, final byte[] data,
      final long dataAddress, final String name) {
    this(instructions, name);
    InvariantChecks.checkNotNull(data);

    this.instructionAddress = instructionAddress;
    this.data = data;
    this.dataAddress = dataAddress;
  }

  public List<IsaPrimitive> getInstructions() {
    return instructions;
  }

  public byte[] getData() {
    return data;
  }

  public long getDataAddress() {
    return dataAddress;
  }

  public long getInstructionAddress() {
    return instructionAddress;
  }

  public String getProgramName() {
    return programName;
  }
}
