/*
 * Copyright 2021 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.simulator;

import ru.ispras.castle.util.FileUtils;
import ru.ispras.microtesk.model.Model;
import ru.ispras.microtesk.options.Options;
import ru.ispras.microtesk.test.GenerationAbortedException;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class ProgramReaderFactory {
  public static BinaryProgramReader getBinaryProgramReader(final Model model, final Options options,
      final String inputProgram) {
    final String extension = FileUtils.getFileExtension(inputProgram).toLowerCase();

    if (ElfProgramReader.FILE_EXTENSION.equals(extension)) {
      return new ElfProgramReader(model, options, inputProgram);
    } else if (BinProgramReader.FILE_EXTENSION.equals(extension)) {
      return new BinProgramReader(model, options, inputProgram);
    } else {
      throw new GenerationAbortedException(
          String.format("Unsupported binary file extension: %s.", extension));
    }
  }
}
