/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.simulator;

import ru.ispras.castle.util.Logger;
import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.microtesk.model.ProcessingElement;
import ru.ispras.microtesk.model.memory.MemoryDevice;

/**
 * The {@link Utils} class provides methods for printing the state of the simulator.
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class Utils {
  public void printMemory(final MemoryDevice mem, long startDataAddress, final int size) {
    startDataAddress >>= 3;

    for (int j = 0; j < size; j++) {
      Logger.debug("Memory (%s[0x%s]): 0x%s", "",
          BitVector.valueOf((startDataAddress + j) << 3, 64).toHexString(),
          mem.load(BitVector.valueOf(startDataAddress + j, 64)).toHexString());
    }
  }

  public void printRegister(final String registerName, final int number,
      final ProcessingElement pe) {
    final MemoryDevice registers = pe.getMemoryDevice(registerName);
    final int size = registers.getAddressBitSize();
    if (number < (1 << size)) {
      Logger.debug("%s[%s]: 0x%s \n", registerName, number,
          registers.load(BitVector.valueOf(number, 64)).toHexString());
    }
  }

  public void printRegisters(final String registerName, final ProcessingElement pe) {
    final MemoryDevice registers = pe.getMemoryDevice(registerName);
    final int size = registers.getAddressBitSize();
    for (int i = 0; i < (1 << size); i++) {
      Logger.debug("%s[%s]: 0x%s \n", registerName, i,
          registers.load(BitVector.valueOf(i, 64)).toHexString());
    }
  }
}
