/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.simulator.memory;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.json.JsonArray;
import javax.json.JsonObject;
import javax.json.JsonReader;
import javax.json.JsonValue;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.microtesk.model.memory.loader.MemoryStructure;
import ru.ispras.microtesk.utils.JsonConfig;

/**
 * The {@code IrMemory} class contains the memory data to be loaded into the simulator's memory
 *
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public abstract class IrMemory {
  public static final String MEMORY_FILE_NAME_TAG = "file_name";

  protected List<Element> elements = new ArrayList<>();

  public static Map<String, String> readMemoryFiles(final Path memoryPaths) {
    InvariantChecks.checkNotNull(memoryPaths);
    JsonReader jsonReader = JsonConfig.getJsonReader(memoryPaths.toString());

    Map<String, String> map = new HashMap<>();

    JsonArray jsonArray = jsonReader.readArray();
    for (JsonValue jsonValue : jsonArray) {
      JsonObject jsonObject = jsonValue.asJsonObject();

      final String memoryStructure = jsonObject.getString(MemoryStructure.TYPE);
      final String memoryFileName = jsonObject.getString(IrMemory.MEMORY_FILE_NAME_TAG);

      map.put(memoryStructure, memoryFileName);
    }

    JsonConfig.close(jsonReader);

    return map;
  }

  public List<Element> getElements() {
    return elements;
  }

  @Override
  public String toString() {
    return String.format("Elements for memory: %s", elements.size());
  }
}
