/*
 * Copyright 2022-2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.simulator.memory;

import java.nio.file.Files;

import ru.ispras.castle.util.Logger;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class MemoryFactory {
  public static IrMemory newMemory(final String modelName, final MemoryFiles memoryFiles) {
    Logger.debug("Path to JSON file containing the program memory files: %s",
        memoryFiles.getMemoryPathsFile());

    if (Files.exists(memoryFiles.getMemoryPathsFile())) {
      return new ArkIrMemory(modelName, memoryFiles);
    } else {
      return newEmptyMemory();
    }
  }

  public static IrMemory newEmptyMemory() {
    return new EmptyMemory();
  }

  private static final class EmptyMemory extends IrMemory {
    private EmptyMemory() {}
  }
}
