package ru.ispras.microtesk.translator.nml.antlrex;

import ru.ispras.fortress.util.InvariantChecks;

import ru.ispras.microtesk.model.data.TypeId;
import ru.ispras.microtesk.translator.nml.ir.shared.Type;

public final class Types {
  public static Type promote(Type source, Type top) {
    InvariantChecks.checkNotNull(top);

    // 'source' is generic integer
    if (source == null) {
      return top;
    }
    return promoteInstance(source, top);
  }

  private static Type promoteInstance(Type source, Type top) {
    if (top.equals(source)) {
      return source;
    }
    // 'top' is non-equal record type, reject
    if (top.getStruct() != null) {
      return null;
    }
    final TypeId tid = source.getTypeId();
    if (!isSubtypeOf(top.getTypeId(), tid)) {
      return null;
    }
    if (tid == TypeId.FLOAT) {
      return null;
    }
    if (source.getBitSize() > top.getBitSize()) {
      return null;
    }
    return top;
  }

  private static boolean isSubtypeOf(TypeId topId, TypeId tid) {
    if (tid == topId) {
      return true;
    }
    TypeId superId = supertypeOf(tid);

    return superId != tid && isSubtypeOf(topId, superId);
  }

  private static TypeId supertypeOf(TypeId tid) {
    switch (tid) {
    case INT:
    case FLOAT:
    case CARD:
      return TypeId.CARD;

    case BOOL:
      return TypeId.INT;
    }
    throw new IllegalStateException();
  }
}
