package ru.ispras.microtesk.translator.nml.ir.shared;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.solver.function.Function;
import ru.ispras.microtesk.translator.nml.antlrex.TypeCast;
import ru.ispras.microtesk.translator.nml.ir.expr.Expr;
import ru.ispras.microtesk.translator.nml.ir.primitive.Primitive;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public final class NmlFunction {
  private final String name;
  private final Map<String, Primitive> params;
  private final Type type;
  private final Function def;

  private Expr body;

  public NmlFunction(String name, Map<String, Primitive> params, Type type) {
    this.name = name;
    this.params = params;
    this.type = type;

    final DataType sort = TypeCast.getFortressDataType(type);
    this.def = new Function(
        name, convertPrimitivesToNodes(params.values()), sort);
  }

  private static List<NodeVariable> convertPrimitivesToNodes(
      Collection<Primitive> items) {
    final var nodes = new ArrayList<NodeVariable>(items.size());
    for (final Primitive p : items) {
      final var node = new NodeVariable(
          p.getName(), TypeCast.getFortressDataType(p.getReturnType()));
      nodes.add(node);
    }
    return nodes;
  }

  public String getName() {
    return name;
  }

  public Map<String, Primitive> getParameters() {
    return params;
  }

  public List<Primitive> getParameterList() {
    return new ArrayList<Primitive>(params.values());
  }

  public Type getValueType() {
    return type;
  }

  public Function getDefinition() {
    return def;
  }

  public Expr getBody() {
    if (!isDefined()) {
      throw new IllegalStateException();
    }
    return body;
  }

  public void setBody(final Expr e) {
    if (isDefined()) {
      throw new IllegalStateException();
    }
    // TODO typecheck
    body = e;
    def.setBody(e.getNode());
  }

  public boolean isDefined() {
    return body != null;
  }
}
