/*
 * Copyright 2022-2023 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.utils;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.json.Json;
import javax.json.JsonReader;
import javax.json.JsonReaderFactory;
import javax.json.JsonWriter;
import javax.json.JsonWriterFactory;
import javax.json.stream.JsonGenerator;

import ru.ispras.castle.util.FileUtils;
import ru.ispras.castle.util.Logger;
import ru.ispras.fortress.util.InvariantChecks;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class JsonConfig{

  public static Map<String, String> getConfig() {
    final Map<String, String> config = new HashMap<>();
    config.put(JsonGenerator.PRETTY_PRINTING, "");
    return config;
  }

  public static JsonWriter getJsonWriter(final String fileName) {
    InvariantChecks.checkNotNull(fileName);

    try {
      final File targetFile = FileUtils.newFile(fileName);
      final FileWriter writer = new FileWriter(targetFile);

      final JsonWriterFactory factory = Json.createWriterFactory(JsonConfig.getConfig());
      Logger.debug("Writer for Json file '%s' was created.", fileName);

      return factory.createWriter(writer);
    } catch (IOException e) {
      Logger.error("An error occurred while creating Json file '%s'.", fileName);

      e.printStackTrace();
    }

    return null;
  }

  public static JsonReader getJsonReader(final String fileName) {
    InvariantChecks.checkNotNull(fileName);

    try {
      final FileReader reader = new FileReader(fileName);

      JsonReaderFactory factory = Json.createReaderFactory(null);
      Logger.debug("Reader for Json file '%s' was created.", fileName);

      return factory.createReader(reader);
    } catch (IOException e) {
      Logger.error("An error occurred while reading Json file '%s'.", fileName);

      e.printStackTrace();
    }

    return null;
  }

  public static void close(JsonWriter jsonWriter) {
    InvariantChecks.checkNotNull(jsonWriter);

    jsonWriter.close();
  }

  public static void close(JsonReader jsonReader) {
    InvariantChecks.checkNotNull(jsonReader);

    jsonReader.close();
  }
}