/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Test;

import ru.ispras.microtesk.tools.adapter.trace.ConcreteEvent;
import ru.ispras.microtesk.tools.adapter.trace.EventElementType;
import ru.ispras.microtesk.tools.adapter.trace.EventType;
import ru.ispras.microtesk.tools.oracle.matcher.EventList;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class EventReaderTestCase {
  @Test
  public void transformationBase() {
    final Map<EventElementType, String> eventElement01 = new HashMap<>();
    eventElement01.put(EventElementType.INSTRUCTION, "div2.64 ->");
    ConcreteEvent concreteEvent01 = new ConcreteEvent(EventType.INSTRUCTION, eventElement01);

    List<ConcreteEvent> concreteEvents = new ArrayList<>();
    concreteEvents.add(concreteEvent01);

    final Map<EventElementType, String> eventElement02 = new HashMap<>();
    eventElement02.put(EventElementType.INSTRUCTION, "binop2 v139");
    ConcreteEvent concreteEvent02 = new ConcreteEvent(EventType.INSTRUCTION, eventElement02);

    concreteEvents.add(concreteEvent02);

    EventList eventReader = new EventList(concreteEvents);

    ConcreteEvent currentEvent = eventReader.getCurrentEvent();
    //Logger.message("ConcreteEvent: '%s'", currentEvent);

    assertEquals(currentEvent.getElement(EventElementType.INSTRUCTION), "div2.64 v139");
  }

  @Test
  public void transformationBase02() {
    final Map<EventElementType, String> eventElement01 = new HashMap<>();
    eventElement01.put(EventElementType.INSTRUCTION, "jne ->");
    ConcreteEvent concreteEvent01 = new ConcreteEvent(EventType.INSTRUCTION, eventElement01);

    List<ConcreteEvent> concreteEvents = new ArrayList<>();
    concreteEvents.add(concreteEvent01);

    final Map<EventElementType, String> eventElement02 = new HashMap<>();
    eventElement02.put(EventElementType.INSTRUCTION, "cond jmp v5, 7");
    ConcreteEvent concreteEvent02 = new ConcreteEvent(EventType.INSTRUCTION, eventElement02);

    concreteEvents.add(concreteEvent02);

    EventList eventReader = new EventList(concreteEvents);

    ConcreteEvent currentEvent = eventReader.getCurrentEvent();
    //Logger.message("ConcreteEvent: '%s'", currentEvent);

    assertEquals(currentEvent.getElement(EventElementType.INSTRUCTION), "jne v5, 7");
  }

  // TODO:
//  public void transformationBase03() {
//    Pattern p = Pattern.compile("([a-z0-9]*, )*[a-z0-9]+$");
//    Matcher m = p.matcher("cond jmp 0x7, v5, v6, u7");
//
//    //Logger.message("m.find(: %s)", m.find());
//    final String part0 = m.group(0);
//    //final String part1 = m.group(1);
//    //Logger.message("part0: '%s'", part0);
//    //Logger.message("part1: '%s'", part1);
//    assertEquals(part0, "0x7, v5, v6, u7");
//  }
}
