/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle.internal;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class RegisterModeTestCase {
  @Test
  public void testSimpleRegisterV() {
    RegisterMode registerMode = new RegisterMode("v([0-9]++)$", "V", false, 0, 65535);

    int index = registerMode.getIndex("v2");
    assertEquals(2, index);

    index = registerMode.getIndex("v003");
    assertEquals(3, index);

    index = registerMode.getIndex("v333");
    assertEquals(333, index);

    index = registerMode.getIndex("v600");
    assertEquals(600, index);

    index = registerMode.getIndex("v0");
    assertEquals(0, index);
  }

  @Test
  public void testSimpleRegisterDv() {
    RegisterMode registerMode = new RegisterMode("DV([0-9]++)$", "R", false, 0, 31);

    int index = registerMode.getIndex("DV2");
    assertEquals(2, index);

    index = registerMode.getIndex("DV03");
    assertEquals(3, index);

    index = registerMode.getIndex("DV31");
    assertEquals(31, index);

    index = registerMode.getIndex("DV6");
    assertEquals(6, index);
  }

  @Test
  public void testSimpleRegisterAbc() {
    RegisterMode registerMode = new RegisterMode("abc$", "ABC", false, 0, 0);

    int index = registerMode.getIndex("abc");
    assertEquals(0, index);
  }

  @Test
  public void testSimpleRegisterMinimips() {
    RegisterMode registerMode = new RegisterMode("[$]([0-9]++)$", "GPR", false, 0, 31);

    int index = registerMode.getIndex("$1");
    assertEquals(1, index);

    index = registerMode.getIndex("$21");
    assertEquals(21, index);

    index = registerMode.getIndex("$07");
    assertEquals(7, index);
  }

  @Test(expected = IllegalArgumentException.class)
  public void testArgumentExceptionIncorrect() {
    RegisterMode registerMode = new RegisterMode("v([0-9]++)$", "V", false, 0, 65535);
    registerMode.getIndex("v333d");
  }

  @Test(expected = IllegalArgumentException.class)
  public void testArgumentExceptionIncorrect2() {
    RegisterMode registerMode = new RegisterMode("v([0-9]++)$", "V", false, 0, 65535);
    registerMode.getIndex("v.333");
  }

  @Test(expected = IllegalArgumentException.class)
  public void testArgumentExceptionIncorrect3() {
    RegisterMode registerMode = new RegisterMode("abc$", "ABC", false, 0, 0);

    registerMode.getIndex("abc2");
  }

  @Test(expected = IllegalArgumentException.class)
  public void testArgumentExceptionTooBig() {
    RegisterMode registerMode = new RegisterMode("v([0-9]++)$", "V", false, 0, 65535);
    registerMode.getIndex("v99992");
  }
}
