/*
 * Copyright 2022 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.tools.oracle.internal;

import static org.junit.Assert.assertEquals;

import java.util.NoSuchElementException;

import org.junit.Before;
import org.junit.Test;

import ru.ispras.castle.util.Logger;

/**
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class RegistersTestCase {
  @Before
  public void prepareLogger() {
    Logger.setDebug(false);
  }

  /**
   * The path to directory containing minimips model.
   */
  private static final String MINIMIPS_MODEL_SOURCE = "/build/target/arch/demo/minimips/";

  /**
   * The path to directory containing file with minimips registers description.
   */
  private static final String MINIMIPS_ORACLE_ITEM_PATH =
      System.getProperty("user.dir") + MINIMIPS_MODEL_SOURCE + Registers.REGISTERS_DESCRIPTION;

  @Test
  public void testSimple() {
    Logger.debug("The path to directory containing minimips items description: %s", MINIMIPS_ORACLE_ITEM_PATH);

    Registers registers = new Registers(MINIMIPS_ORACLE_ITEM_PATH);
    Item registerMode = registers.recognizeItem("$1");
    Logger.debug("%s", registerMode);

    int index = registerMode.getIndex("$1");
    assertEquals("Recognized register must be 1", 1, index);

    index = registerMode.getIndex("$21");
    assertEquals("Recognized register must be 21", 21, index);

    index = registerMode.getIndex("$07");
    assertEquals("Recognized register must be 7", 7, index);
  }

  @Test(expected = NoSuchElementException.class)
  public void testNoSuchElementException() {
    Registers registers = new Registers(MINIMIPS_ORACLE_ITEM_PATH);
    registers.recognizeItem("abc2");
  }
}
